#ifndef _FBO_H_
#define _FBO_H_

// velkost shadowmap
#define SHADOWMAPSIZE 512

// 1 depth, 2 frame
struct FBO0 {
 GLuint id,depth,tex_id[2];
};

// 1 depth, 1 frame
struct FBO1 {
 GLuint id,depth,tex_id;
};

// 0 depth, 3 frame
struct FBO2 {
 GLuint id,tex_id[3];
};

// 1. frame - depth, 2. frame - normal, 3. frame -color
struct FBO_SSAO {
 GLuint id,depth,normal,color;
};

// 1 depth
struct FBO_SHADOWMAP {
 GLuint id,depth;
};

// 1. frame - depth_stencil, 2. frame - albedo, 3. frame - normal,
// 4. frame - depth, 5. frame - pohybove vektory
struct FBO_RAW {
 GLuint id,tex_id[5];
};

// trieda premennych a funkcii pre pracu s OpenGL FBO
class CFBO {
public:
 CFBO();  // konstruktor
 ~CFBO(); // destruktor

 int init0(void);            // inicializacia FBO0
 int init(void);             // inicializacia FBO1 a FBO2
 int init_downsampler(void); // inicializacia FBO pre downsampler
 int init_ssao(void);        // inicializacia FBO pre SSAO
 int init_shadowmap(void);   // inicializacia FBO pre shadowmap
 int init_raw(void);         // inicializacia FBO pre nespracovane udaje
 void attach_layer(int L);   // pripojenie L-tej vrstvy 3D textury
 void cancel(void);          // zrusenie FBO
 void drawfbo1_attachment(void);  // vykreslenie obsahu FBO1
 void drawfbo2_attachment(int N); // vykreslenie obsahu FBO2
 void draw(void); // vykreslenie objektu FBO

 GLuint getfbo0(void) { return(f0.id); }
 GLuint getfbo1(void) { return(f1.id); }
 GLuint getfbo2(void) { return(f2.id); }
 GLuint getfbo_ssao(void) { return(f_ssao.id); }
 GLuint getfbo_shadowmap(void) { return(f_shadowmap.id); }
 GLuint getfbo_raw(void) { return(f_raw.id); }
 GLuint getid(int N) { return(f_downsampler[N].id); }
 GLuint getfbo0_tex_id(int N) { return(f0.tex_id[N]); }
 GLuint getfbo1_tex_id(void) { return(f1.tex_id); }
 GLuint getfbo2_tex_id(int N) { return(f2.tex_id[N]); }
 GLuint gettex_id(int N) { return(f_downsampler[N].tex_id); }
 GLuint ssao_depth(void) { return(f_ssao.depth); }
 GLuint ssao_normal(void) { return(f_ssao.normal); }
 GLuint ssao_color(void) { return(f_ssao.color); }
 GLuint shadowmap_depth(void) { return(f_shadowmap.depth); }
 GLuint getfboraw_tex_id(int N) { return(f_raw.tex_id[N]); }
private:
 FBO0 f0; // FBO0 pre src_w,scr_h
 FBO1 f1; // FBO1 pre scr_w,scr_h
 FBO2 f2; // FBO2 pre scr_w2,scr_h2

 FBO1 f_downsampler[4]; // 4 FBO pre downsampler

 FBO_SSAO f_ssao; // FBO pre SSAO, rozmery: scr_w x scr_h

 FBO_SHADOWMAP f_shadowmap; // FBO pre shadowmap, rozmery: SHADOWMAPSIZE x SHADOWMAPSIZE

 FBO_RAW f_raw; // FBO pre nespracovane udaje (deferred shading)

 GLenum ret; // pomocne premenne
 int created;

 GLuint v_buf,t_buf; // zasobniky pre vrcholy a texturovacie suradnice
 GLuint vao; // vertex array object
 float v[18]; // 3x6 vrcholov stvoruholnika
 float t[12]; // 2x6 texturovacich suradnic

 // inicializacia stvoruholnika predstavujuceho FBO zlozeny z 2 trojuholnikov
 void init_obj(void);
};

#endif

/*
FRAMEBUFFER_COMPLETE_EXT                          0x8CD5
FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT             0x8CD6
FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT     0x8CD7
FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT             0x8CD9
FRAMEBUFFER_INCOMPLETE_FORMATS_EXT                0x8CDA
FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT            0x8CDB
FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT            0x8CDC
FRAMEBUFFER_UNSUPPORTED_EXT                       0x8CDD
*/
