#ifndef _TEXTURE_H_
#define _TEXTURE_H_

// praca s texturami *******************************************************

// maximalny pocet textur
#define MAX_TEX_NUM 256

// volitelne parametre textury
enum {
 T_DEFAULT=1<<0,
 T_CLAMP=1<<1,
 T_CLAMP_TO_EDGE=1<<2,
 T_NEAREST=1<<3,
 T_LINEAR=1<<4,
 T_MIPMAP=1<<5
};

// steny CubeMap
enum CUBE_FACE {
 CUBE_FACE_POSITIVE_X,
 CUBE_FACE_NEGATIVE_X,
 CUBE_FACE_POSITIVE_Y,
 CUBE_FACE_NEGATIVE_Y,
 CUBE_FACE_POSITIVE_Z,
 CUBE_FACE_NEGATIVE_Z,
};

// trieda premennych a funkcii pre pracu s OpenGL texturami
class CTexture {
public:
 CTexture();  // konstruktor
 ~CTexture(); // destruktor

 GLuint tex_count; // pocet nahratych textur

 GLuint tex_id[MAX_TEX_NUM]; // identifikatory nahratych textur

 int load2D(GLuint ID,const char *F,GLuint MODE); // nahratie 2D textury
 int loadCubeMap(GLuint ID,const char *F);        // nahratie CubeMap textury

 int create_grayscale(GLuint ID); // vytvorenie grayscale 2D textury

 int prepare_perlin_noise_tex(GLuint ID); // priprava 3D textury (Perlinov sum)

 int prepare_random_vec_tex(GLuint ID); // priprava textury s nahodne orientovanymi vektormi

 void freeTexture(GLuint ID); // zrusenie textury
private:
 // SDL nacitanie suboru s obrazkom textury
 SDL_Surface *img;

 // pomocna pamat
 GLubyte *data;

 // pocet bytov na texel (bod textury)
 GLuint bpp;
 // format a interny format udajov ulozenych v texture (na zaklade bpp)
 GLuint glFormat,intFormat;

 // pomocne premenne pre pripravu 3D textury Perlinovho sumu
 float r,freq,ampl,col;
 float i1,i2,i3,i4;
 int c,ix,iy,iz;

 char *clrs;

 // uprava farieb bodov textury na zaklade typu suboru
 int correctData(SDL_Surface *S);
 // nastavenie parametrov a vytvorenie 2D textury
 void genTexture(GLuint W,GLuint H,GLubyte *DATA,GLuint MODE);

 // pseudonahodne realne cislo z intervalu <-1.0f;1.0f>
 float randnum(int X,int Y,int Z) {
  // vypocet pseudonahodneho cisla pre 3D suradnicu X,Y,Z
  // 23, 53 - prvocisla
  c=X+Y*23+Z*53;
  // 32b signed int - cele cislo od -2147483648 do 2147483647 => (+-) 2^31
  // bitovy posuv (32b int) c o 13 miest dolava => dojde k strate niekolkych MSB bitov 
  // ^ = XOR => 0 0 = 0, 0 1 = 1, 1 0 = 1, 1 1 = 0
  c^=(c<<13);
  // 15731, 789221 - prvocisla
  // &0x7fffffff = AND => c AND 00000111 | 11111111 | 11111111 | 11111111 =>
  // zostane iba 27 LSB bitov, cislo bude vzdy kladne v intervale <0.0f,2.0f>
  // odcitanie od 1.0f umiestni cislo do intervalu <-1.0f,1.0f>
  r=(1.0f-(((c*(c*c*15731+789221)+1376312589)&0x7fffffff)/1073741824.0f));
  return(r);
 }
 // interpolacia 2 realnych cisiel
 float interpolate(float A,float B,float F) {
  // linear (lerp) (1.0f-F)*A+F*B = A+(B-A)*F
  // return(A+(B-A)*F);
  // cos
  r=(1.0f-cos(F*PI))*0.5f;
  return(A*(1-r)+B*r);
 }
 // vyhladene pseudonahodne cislo
 float smooth_randnum(int X,int Y,int Z);
 // trilinearna = 3D interpolacia pseudonahodnych cisiel
 float trilinear_randnum(float X,float Y,float Z);
 // Perlinov sum
 float perlin_noise(float X,float Y,float Z);
};

#endif
