#version 130 // GLSL verzia

uniform sampler2D depth_buf_unit,normal_buf_unit,rndvec_buf_unit,base_tex_unit;
uniform vec3 samples[16];
uniform mat4x4 sbproj,isbproj;

// prenasane udaje
in vec2 uv0,uv1;

// hlbka, ambient komponent,vzdialenost
float depth,ambient,dist;

// poloha aktualneho fragmentu, poloha vzorky, farba
vec4 pos,sampleproj,sampleunproj,color;
vec3 samplepos;

// normala, vektor, tangenta, bitangenta, pomocny vektor
vec3 N,V,T,B,vv;

// rotacna matica
mat3x3 rot;

void main()
{
 // hlbka fragmentu ulozena v 2D texture
 depth=texture(depth_buf_unit,uv0).r;

 ambient=0.0;

 // vylucenie pozadia sceny
 if (depth<1.0) {
  // vypocet 3D polohy fragmentu v EYE space zo znamej hlbky a 2D polohy
  // poloha v CLIP space = SCALE-BIAS * PROJECTION * poloha v EYE space
  // poloha v EYE space = iPROJECTION * iSCALE-BIAS * poloha v CLIP space
  pos=isbproj*vec4(uv0,depth,1.0); pos.xyz/=pos.w;

  // normala z 2D textury
  N=normalize(texture(normal_buf_unit,uv0).rgb*2.0-1.0);
  // nahodne orientovany vektor z 2D textury
  V=normalize(texture(rndvec_buf_unit,uv1).rgb*2.0-1.0);

  T=normalize(V-N*dot(V,N)); // tangent - povodne vektor kolmy na N, v tomto pripade nahodne zrotovany
  B=cross(N,T);              // bitangent - vektor kolmy na N a T

  rot=mat3x3(T,B,N); // rotacna matica

  // pre 16 vzoriek
  for (int i=0;i<16;i++) {
   // poloha vzorky v EYE space
   samplepos=rot*samples[i]+pos.xyz;
   // poloha vzorky v CLIP space
   sampleproj=sbproj*vec4(samplepos,1.0); sampleproj.xyz/=sampleproj.w;

   // hlbka fragmentu ulozena v 2D texture na pozicii vzorky
   depth=texture(depth_buf_unit,sampleproj.st).r;

   // ak je hlbka fragmentu mensia ako hlbka vzorky, fragment
   // "brani" prieniku svetla => znizuje ambient komponent
   if (depth<sampleproj.z) {
    // vypocet 3D polohy vzorky v EYE space zo znamej hlbky a 2D polohy
    sampleunproj=isbproj*vec4(sampleproj.st,depth,1.0); sampleunproj.xyz/=sampleunproj.w;

    vv=sampleunproj.xyz-pos.xyz; // vektor fragment -> vzorka
    dist=dot(vv,vv);             // vzdialenost ^2

    ambient+=1.0/(1.0+dist);     // blizsia vzorka ma vacsi vplyv ako vzdialenejsia
   }
  }

  // ambient komponent osvetlenia je znizeny v pripade ak je
  // aktualne spracovavany fragment ovplyvneny inym fragmentom
  ambient=1.0-ambient*0.08;
 }

 // farba zo zakladnej textury
 color=texture(base_tex_unit,uv0);
 // vysledna farba
 gl_FragData[0]=color*vec4(ambient,ambient,ambient,1.0);
}
